#include <Wire.h> //Für Adafruit_BMP085.h
#include <SPI.h> //Für WiFi.h
#include <WiFi.h>
#include <Adafruit_BMP085.h>
#include <dht.h>

char ssid[] = "VDSoftware";    // der Netzwerk-Name
char pass[] = "Jammertal";     // Das Netzwerk-Passwort
int keyIndex = 0;              // your network key Index number (needed only for WEP)
int status = WL_IDLE_STATUS;
WiFiServer server(80);

Adafruit_BMP085 bmp;
const int DHT11_PIN = A0;
dht DHT;
int dht_zustand;

float temperatur_bmp = 0.0f;
float temperatur_dht = 0.0f;
float luftfeuchtigkeit = 0.0f;
float luftdruck = 0.0f;
float temperatur = 0.0f;

float temperatur_maximum = -1000.0f;
float temperatur_minimum = 1000.0f;
float luftfeuchtigkeit_maximum = 0.0f;
float luftfeuchtigkeit_minimum = 100.0f;
float luftdruck_maximum = 0.0f;
float luftdruck_minimum = 10000000.0f;


void setup() {
  Serial.begin(9600);
  while (!Serial) { } //Wird für den Ardiono Leonardo benötigt

  if (!bmp.begin()) {Serial.println("Kein BMP085 gefunden!"); }

  if (WiFi.status() == WL_NO_SHIELD) {
    Serial.println("WiFi shield konnte nicht gefunden werden!");
    while (true); //Mache sonst nichts
  }

  if ( WiFi.firmwareVersion() != "1.1.0" )
    Serial.println("Firmware ist veraltet!");

  //Versuche die Verbindung zum WLAN herzustellen
  while ( status != WL_CONNECTED) {
    Serial.print("Verbindungsversuch mit SSID: ");
    Serial.println(ssid);  
    status = WiFi.begin(ssid, pass);
    Serial.println("Warte 10 Sekunden auf eine Verbindung...");
    delay(10000);
  }
  server.begin();
  printWifiStatus(); //Gebe aus wie IP und Netzwerkname lauten
}


void loop() {
  WiFiClient client = server.available(); 
  if (client) { //Hat sich ein Client verbunden?
    Serial.println("Neuer Client");
    boolean nichts_geschrieben = true; //Hat der Client desletzt etwas geschrieben?
    while (client.connected()) { //Solange der Client verbunden ist
      if (client.available()) {  //und verfügbar ist
        char c = client.read();  //Schaue nach was er geschrieben hat
        
        if (c == '\n' && nichts_geschrieben) { //Er hat 2mal nichts geschrieben, wir dürfen antworten
          // send a standard http response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close"); //Beende die Verbindung nach dem Laden der Seite
          client.println("Refresh: 30"); //Lade die Seite alle 30 Sekunden neu
          client.println();
          client.println("<!DOCTYPE HTML>");
          
          daten_updaten();
          seite_aufbauen(client);
          
          break;
        }
        if (c == '\n') {             //Der Client hat nichts geschrieben
          nichts_geschrieben = true; //Nächstes mal dürfen wir antworten
        } else if (c != '\r') {     //Er hat etwas geschrieben
          nichts_geschrieben = false;//Wir dürfen nächstes mal nicht antworten
        }
      }
    }
    delay(1); //Warte, bis der WebBrowser die Daten empfangen hat

    client.stop(); //Beende die Verbindung
  }
}

void daten_updaten() {
  dht_zustand = DHT.read11(DHT11_PIN); //Lese den DHT-Sensor aus
  luftfeuchtigkeit = DHT.humidity; 
  luftfeuchtigkeit_maximum = max(luftfeuchtigkeit, luftfeuchtigkeit_maximum);
  luftfeuchtigkeit_minimum = min(luftfeuchtigkeit, luftfeuchtigkeit_minimum);
  temperatur_dht = DHT.temperature; //Lese die Temperatur des DHT11 aus
  temperatur_bmp = bmp.readTemperature(); //Lese die Temperatur des BMP085 aus
  temperatur = temperatur_dht; //Benutze die Temperatur des DHT11
  temperatur_maximum = max(temperatur, temperatur_maximum);
  temperatur_minimum = min(temperatur, temperatur_minimum);
  luftdruck = bmp.readPressure();  //Lese den Luftdruck des BMP085 aus
  luftdruck_maximum = max(luftdruck, luftdruck_maximum);
  luftdruck_minimum = min(luftdruck, luftdruck_minimum);  
}

void seite_aufbauen(WiFiClient client) {
  client.print("<html>");
  
  client.print("<h2> Wetterstation </h2>");                   //Die Überschrift
  
  client.print("<b> Temperatur      </b>");                   //Temperatur
  client.print(temperatur);
  client.print(" &deg;C    <font color=\"LightGray\"> min: ");//min und max in Grauer schrift
  client.print(temperatur_minimum);
  client.print(" &deg;C   max: ");
  client.print(temperatur_maximum);
  client.print(" &deg;C</font><br />");
  
  client.print("<b>Luftfeuchtigkeit  </b>");                  //Luftfeuchtigkeit
  client.print(luftfeuchtigkeit);
  client.print(" %    <font color=\"LightGray\"> min: ");     //min und max in Grauer schrift
  client.print(luftfeuchtigkeit_minimum);
  client.print(" %   max: ");
  client.print(luftfeuchtigkeit_maximum);
  client.print(" %</font><br />");
  
  client.print("<b> Luftdruck     </b>");                     //Lutdruck
  client.print(luftdruck / 100.0f);
  client.print(" HPa    <font color=\"LightGray\"> min: ");    //min und max in Grauer schrift
  client.print(luftdruck_minimum / 100.0f);
  client.print(" HPa   max: ");
  client.print(luftdruck_maximum / 100.0f);
  client.print(" HPa</font><br />");
  
  client.println("</html>");
}

void printWifiStatus() {
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID()); //Der Netzwerkname

  Serial.print("IP Addresse: ");
  Serial.println(WiFi.localIP()); //Die IP
  //Unter dieser IP ist die Wetterstation vom WLAN aus erreichbar.
}
